step "setting up /target directory for bexbackend"
mkdir -p /etc/bexng

cat >"/etc/bexng/secrets.exs" <<EOF
[
  authy_url: "${AUTHY_URL}",
  authy_key: "${AUTHY_KEY}",

  identitymind_url: "${IDENTITYMIND_URL}",
  identitymind_id: "${IDENTITYMIND_ID}",
  identitymind_secret: "${IDENTITYMIND_SECRET}",

  mandrill: "${MANDRILL_KEY}",

  admin: "${BEXNG_DEFAULT_ADMIN_EMAIL}",
  admin_password: "${BEXNG_DEFAULT_ADMIN_PASSWORD}",


  coin_networks: [:"${WALLET_NETWORK}"],

  wallets: [
    {"${WALLET_NAME}", [
      network: "${WALLET_NETWORK}",
      private_key: "${WALLET_PRIVATE_KEY}",
      public_key: "${WALLET_PUBLIC_KEY}",
      chain_code: "${WALLET_CHAIN_CODE}"
    ]}
  ]
]
EOF

step_done


step "updating docker basefiles"
gem update 'docker-utils' --no-rdoc --no-ri
docker pull 'bexio/base:latest' >/dev/null 2>&1
docker pull 'bexio/toolchain:elixir-upgrade' >/dev/null 2>&1
step_done


start_bitcoind() {

if [ "${WALLET_NETWORK}" = "testnet3" ]; then
  bitcoin_data_volume='tbtc-data'
else
  bitcoin_data_volume='btc-data'
fi

docker pull 'scratch:latest' >/dev/null 2>&1

docker run \
  --name='bitcoind-data' \
  --volumes-from="${bitcoin_data_volume}" \
  --entrypoint 'volume' \
  'scratch:latest' || :

step "starting bitcoind"

docker pull 'bexio/bitcoind:latest' >/dev/null 2>&1

docker run -d --restart=always \
  --name='bitcoind' \
  -e "NETWORK=${WALLET_NETWORK}" \
  --volumes-from='bitcoind-data' \
  'bexio/bitcoind:latest'

step_done
}


start_db() {
docker pull 'bexio/amb:latest' >/dev/null 2>&1

step "starting db"

docker run -d \
  --name='rds-postgres-1' \
  "bexio/amb:latest" \
  "${DB_HOST}" "${DB_PORT}"

docker-utils block-until-port-open 'rds-postgres-1' '5432'

docker pull 'bexio/pgproxy:latest' >/dev/null 2>&1

docker run --rm -i -t \
  --name='bexng-db-init-task' \
  --link="rds-postgres-1:postgresql" \
  -e "NO_PROXY=1" \
  "bexio/pgproxy:latest" \
  "${DB_ADMIN_USERNAME}" "${DB_ADMIN_PASSWORD}" \
  'bexng' "${BEXNG_DB_SECRET}"

docker kill 'rds-postgres-1'
docker rm 'rds-postgres-1'

docker run -d --restart=always \
  --name='rds-postgres' \
  "bexio/amb:latest" \
  "${DB_HOST}" "${DB_PORT}"

step_done


}


start_armoryd() {
step "starting armoryd"

docker pull 'bexio/armoryd:latest' >/dev/null 2>&1

docker run -d --restart=always \
  --name="armoryd" \
  -e "NETWORK=${WALLET_NETWORK}" \
  "bexio/armoryd:latest"

step_done
}

start_bexbackend() {
docker-utils block-until-port-open 'rds-postgres' '5432'
docker-utils block-until-port-open 'bitcoind' '8333'
docker-utils block-until-port-open 'armoryd' '80'

step "starting bexbackend"

docker pull "bexio/bexng:${BACKEND_IMAGE_TAG}" >/dev/null 2>&1

export DEPLOY_TARGET="${DOMAIN}"

docker run -d --restart=always \
  --name='bexbackend' \
  --link='rds-postgres:postgresql' \
  --link='bitcoind:bitcoind' \
  --link='armoryd:armoryd' \
  --volume "/etc/bexng:/target" \
  --publish="51607:51607" \
  -e "BEXNG_DB_SECRET" \
  -e "DEPLOY_TARGET" \
  "bexio/bexng:${BACKEND_IMAGE_TAG}"

step_done
}

start_bexfrontend() {
docker-utils block-until-port-open 'bexbackend' '51607'

step "starting bexfrontend"

docker pull "bexio/frontend:${FRONTEND_IMAGE_TAG}" >/dev/null 2>&1

docker run -d --restart=always \
  --name='bexfrontend' \
  --link="bexbackend:backend" \
  -e 'BEXNG_PORT_51608_TCP=tcp://backend:51608' \
  --publish='80:8081' \
  "bexio/frontend:${FRONTEND_IMAGE_TAG}"

step_done
}


start_bitcoind &
start_db &
start_armoryd &
start_bexbackend &
start_bexfrontend &
wait

