die(){
  echo "error: $1" >&2
  exit 1
}

set -e

export HOME=/root
cd "$HOME"

echo "enabling mirroring for security apt sources"
sed -i "s/security\.ubuntu\.com/${AWS_REGION}.ec2.archive.ubuntu.com/g" /etc/apt/sources.list

echo "installing packages"
afg() {
  DEBIAN_FRONTEND=noninteractive \
  apt-get -q -y --force-yes \
  -o Dpkg::Options::="--force-confnew" \
  $@
}

afg clean
afg update
afg dist-upgrade
afg install htop tree btrfs-tools xz-utils kexec-tools debconf-utils rsyslog-gnutls
afg install python-setuptools python-pip python-setproctitle bundler ruby-nokogiri
afg clean

debconf-set-selections <<EOF
kexec-tools kexec-tools/load_kexec boolean true
kexec-tools kexec-tools/use_grub_config boolean true
EOF


pip install -q awscli
gem install -q --no-rdoc --no-ri aws-sdk docker-utils

echo "installing aws cfn-tools"
easy_install "https://s3.amazonaws.com/cloudformation-examples/aws-cfn-bootstrap-latest.tar.gz"

provision_volume() {
  device="$1"; shift
  mntpt="$1"; shift

  mkfs.btrfs "${device}" 2>/dev/null || :

  mount "${device}" /mnt 2>/dev/null || :
  btrfs filesystem resize max /mnt 2>/dev/null || :
  umount /mnt 2>/dev/null || :

  mkdir -p "${mntpt}"
  mount -t btrfs -o "rw,noatime,space_cache" "${device}" "${mntpt}"
}

if [ -b "/dev/xvdc" ]; then
  provision_volume "/dev/xvdc" "/var/lib/docker"
fi

if [ -b "/dev/xvdd" ]; then
  provision_volume "/dev/xvdd" "/var/lib/docker/vfs/dir"
fi

if ! hash docker; then
  echo "installing docker..."
  echo 'deb http://get.docker.io/ubuntu docker main' > /etc/apt/sources.list.d/docker.list
  apt-key adv --keyserver 'hkp://pgp.mit.edu:80' --recv-keys '36A1D7869245C8950F966E92D8576A8BA88D21E9'
  apt-get update
  apt-get install -qy lxc-docker
  gpasswd -a ubuntu docker
fi

echo "logging into ${DOCKER_REGISTRY_URL}..."
cat >"$HOME/.dockercfg" <<EOF
{"${DOCKER_REGISTRY_URL}": {
  "auth": "${DOCKER_REGISTRY_AUTH}",
  "email": "${DOCKER_REGISTRY_EMAIL}"
}}
EOF

docker pull tianon/true:latest
docker pull bexio/base:latest

docker pull bexio/amb:latest
docker pull bexio/bitcoind:latest
docker pull bexio/armoryd:latest
docker pull bexio/postgresql:latest
docker pull bexio/bexng:latest
docker pull bexio/bexng-frontend:latest
docker pull bexio/toolchain:latest

echo "bootstrapping bitcoind service"
docker run --name="btc-data" bexio/bitcoind:latest /bin/true
docker run --name="tbtc-data" bexio/bitcoind:latest /bin/true
